﻿
; --------------------------------------------------------------------------------------------------------------
; IO.pbi
; Author: cxalex @ purebasic.fr/german
; Version: 0.3
; --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; Changelog:
; # 0.3
;   - Added: IO\Truncate()
;   - Added: IO\IOHandle() to get the internal handle
;   - Added: SerialPort Implementation
;   - Added: Versioning
;   - Changed: EnableExplicit only active if IO.pbi is main file
;
; # 0.2 :
;   - Added: Network Implementation
;   - BugFix: Released __IO Structure correct
;
; # 0.1 :
;   - Initial Release
;} --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; ToDo:
;   - Gadget
;   - Program
;   - Console
;} --------------------------------------------------------------------------------------------------------------

CompilerIf #PB_Compiler_IsMainFile
  EnableExplicit
CompilerEndIf

; IO - Default Interface
Interface IO
  ; Obligatory:
  Write(*Mem, MemSize.i) ; Writes memory to IO
  Read(*Mem, MemSize.i) ; Reads memory from IO
  IsAvailable()          ; Checks how Mmny Data is available at IO between data-pointer and end-of-data
  Truncate()             ; Cuts of all data after data-pointer
  Close()                ; Closes IO
  ; Return Handle for IO Object
  IOHandle()
  ; Returns Name of IO - Port
  IOName$()
  ; Check Availability of optional Functions
  IOOptionalFunctions()
  ; Optional
  GetLength()               ; Get total lenght of data in IO - object
  GetPosition()             ; Sets data-pointer-position 
  SetPosition(Position.i)   ; Get data-pointer-position
  ImportFromIO(*IO.IO)      ; Imports data from IO - object
  ExportToIO(*IO.IO)        ; Exports data to IO - object
EndInterface

; IO - Data Holder
Structure __IO
  *vTable ; vTable
  *handle ; Internal Handle for Specific Implementation
  optionalFunctionality.i ; Optional Functionality
  name$ ; Name of the implementation
EndStructure

Enumeration
  #IO_GetLenght        = 1<<0 ; Can request total legth of IO - object
  #IO_GetPosition      = 1<<1 ; Gets current position in IO - object
  #IO_SetPosition      = 1<<2 ; Sets current postion in IO - object
  #IO_ImportFromIO     = 1<<3 ; Replaces all data with data from IO - object. Imported object must support ExportToIO
  #IO_ExportToIO       = 1<<4 ; Exports all available data to a IO - object
EndEnumeration

; All: Returns optional implemented functions
Procedure __IO_All_GetOptionalFunctions(*IO.__IO)
  ProcedureReturn *IO\optionalFunctionality
EndProcedure

; All: Returns name
Procedure$ __IO_All_Name(*IO.__IO)
  ProcedureReturn *IO\name$
EndProcedure

; All: Returns Handle
Procedure __IO_All_Handle(*IO.__IO)
  ProcedureReturn *IO\handle
EndProcedure

; Include Implementations:
XIncludeFile "IO_Implementation_File.pbi"
XIncludeFile "IO_Implementation_Memory.pbi"
XIncludeFile "IO_Implementation_Network.pbi"
XIncludeFile "IO_Implementation_SerialPort.pbi"


; IDE Options = PureBasic 5.11 (Windows - x86)
; CursorPosition = 12
; Folding = -
; EnableXP
; CPU = 1