﻿
; --------------------------------------------------------------------------------------------------------------
; IO.pbi : Implementation: Memory
; Author: cxalex @ purebasic.fr/german
; Version: 0.1
; --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; Changelog:
; # 0.1 :
;   - Initial Release
;} --------------------------------------------------------------------------------------------------------------

; EnableExplicit

Structure GMem_Data
  *Mem
  MemSize.i
  cPos.i
  BlockCount.i
  BlockSize.i
EndStructure

; IO: Memory
Procedure IO_Memory(BlockSize = 512)
  Protected *GMem.GMem_Data = AllocateMemory(SizeOf(GMem_Data))
  *GMem\BlockSize = BlockSize
  Protected *IO.__IO = AllocateMemory(SizeOf(__IO))
  *IO\vTable = ?__vTable_IO_Memory
  *IO\name$ = "MEMORY"
  *IO\handle = *GMem
  *IO\optionalFunctionality = (#IO_GetLenght|#IO_GetPosition|#IO_SetPosition|#IO_ImportFromIO|#IO_ExportToIO)
  ProcedureReturn *IO
EndProcedure

; Memory: Writer
Procedure __IO_Memory_Write(*IO.__IO, *Mem, MemSize.i)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    While (MemSize + \cPos) > (\BlockCount*\BlockSize)
      \BlockCount+1
      \Mem = ReAllocateMemory(\Mem, \BlockCount*\BlockSize)
    Wend
    
    \Mem = ReAllocateMemory(\Mem, \MemSize + MemSize)
     CopyMemory(*Mem, \Mem + \cPos, MemSize)
     \MemSize + MemSize
     \cPos + MemSize 
  EndWith
EndProcedure

; Memory: Reader
Procedure __IO_Memory_Read(*IO.__IO, *Mem, MemSize.i)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
     CopyMemory(\Mem + \cPos, *Mem,  MemSize)
     \cPos + MemSize 
  EndWith
EndProcedure

; Checks Available Data
Procedure __IO_Memory_IsAvailable(*IO.__IO, Seek.i)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    ProcedureReturn \MemSize - \cPos
  EndWith
EndProcedure

; Memory, GetLenght
Procedure __IO_Memory_GetLength(*IO.__IO)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    ProcedureReturn \MemSize
  EndWith
EndProcedure

; Memory, GetPosition
Procedure __IO_Memory_GetPosition(*IO.__IO, Seek)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    ProcedureReturn \cPos
  EndWith
EndProcedure

; Memory, SetPosition
Procedure __IO_Memory_SetPosition(*IO.__IO, Seek)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    \cPos = Seek
  EndWith
EndProcedure

; Memory, Import From IO
Procedure __IO_Memory_ImportFromIO(*IO.__IO, *IO2.IO)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    If *IO2\IOOptionalFunctions() & #IO_ExportToIO
      ; Delete all Data in Memory
      If \Mem
        FreeMemory(\Mem)
        \BlockCount = #Null
        \cPos = #Null
        \MemSize = #Null
        \Mem = #Null
      EndIf
      ; Exports all Data from IO2 to IO
      *IO2\ExportToIO(*IO)
      ProcedureReturn #True
    EndIf
  EndWith
EndProcedure

; Memory, Import From IO
Procedure __IO_Memory_ExportToIO(*IO.__IO, *IO2.IO)
  Protected *GMem.GMem_Data = *IO\handle
  *IO2\Write(*GMem\Mem, *GMem\MemSize)
EndProcedure

; Memory, Truncate
Procedure __IO_Memory_Truncate(*IO.__IO)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    \MemSize = \cPos
  EndWith
EndProcedure

; Memory, Close
Procedure __IO_Memory_Close(*IO.__IO)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    If \Mem
      FreeMemory(\Mem) 
    EndIf
    FreeMemory(*GMem)
  EndWith
  ClearStructure(*IO, __IO)
  FreeMemory(*IO)
EndProcedure

; Memory: Handle
Procedure __IO_Memory_Handle(*IO.__IO)
  Protected *GMem.GMem_Data = *IO\handle
  With *GMem
    ProcedureReturn \Mem
  EndWith
EndProcedure

; VTable: Memory
DataSection
  __vTable_IO_Memory:
  Data.i @__IO_Memory_Write()
  Data.i @__IO_Memory_Read()
  Data.i @__IO_Memory_IsAvailable()
  Data.i @__IO_Memory_Truncate()
  Data.i @__IO_Memory_Close()
  Data.i @__IO_Memory_Handle()
  Data.i @__IO_All_Name()
  Data.i @__IO_All_GetOptionalFunctions()
  Data.i @__IO_Memory_GetLength()
  Data.i @__IO_Memory_GetPosition()
  Data.i @__IO_Memory_SetPosition()
  Data.i @__IO_Memory_ImportFromIO()
  Data.i @__IO_Memory_ExportToIO()
EndDataSection

; IDE Options = PureBasic 5.11 (Windows - x86)
; CursorPosition = 153
; FirstLine = 126
; Folding = ---
; EnableXP
; CPU = 1