﻿
; --------------------------------------------------------------------------------------------------------------
; IO.pbi : Implementation: Network
; Author: cxalex @ purebasic.fr/german
; Version: 0.2
; --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; Changelog:
; # 0.2 :
;   - Initial Release
;} --------------------------------------------------------------------------------------------------------------

; EnableExplicit

Structure __NetConnection
  Handle.i
  *Buffer
  BufferSize.i
  BufferPos.i
  ReceiveSize.l
  IsConnection.a
EndStructure

; Network: Open Connection
Procedure IO_Network_Connection(ServerName$, Port, Mode = #PB_Network_TCP|#PB_Network_IPv4, TimeOut = 5000, LocalIP$ = "", LocalPort = #Null)
  Protected Connection = OpenNetworkConnection(ServerName$, Port, Mode, TimeOut, LocalIP$, LocalPort)
  If Connection
    Protected *IO.__IO = AllocateMemory(SizeOf(__IO))
    Protected *NCon.__NetConnection = AllocateMemory(SizeOf(__NetConnection))
    *NCon\Handle = Connection
    If Bool(Mode & #PB_Network_UDP)
      *NCon\ReceiveSize = 2048
    Else
      *NCon\ReceiveSize = 65536
    EndIf
    *NCon\IsConnection = #True
    *IO\handle = *NCon
    *IO\name$ = "NETWORK:CONNECTION"
    *IO\optionalFunctionality = (#IO_ImportFromIO|#IO_ExportToIO)
    *IO\vTable = ?__vTable_IO_Network
  EndIf
  ProcedureReturn *IO
EndProcedure

; Network: Handle Client at Server
Procedure IO_Network_Client(ClientID, Mode = #PB_Network_TCP)
  Protected *IO.__IO = AllocateMemory(SizeOf(__IO))
  Protected *NCon.__NetConnection = AllocateMemory(SizeOf(__NetConnection))
  *NCon\Handle = ClientID
  If Bool(Mode & #PB_Network_UDP)
    *NCon\ReceiveSize = 2048
  Else
    *NCon\ReceiveSize = 65536
  EndIf
  *NCon\IsConnection = #False
  *IO\handle = *NCon
  *IO\name$ = "NETWORK:CLIENT"
  *IO\optionalFunctionality = (#IO_ImportFromIO|#IO_ExportToIO)
  *IO\vTable = ?__vTable_IO_Network
  ProcedureReturn *IO
EndProcedure

; Network, Write
Procedure __IO_Network_Write(*IO.__IO, *Mem, MemSize.i)
  Protected *NCon.__NetConnection = *IO\handle 
  ProcedureReturn SendNetworkData(*NCon\Handle, *Mem, MemSize)
EndProcedure

; Network, Read
Procedure __IO_Network_Read(*IO.__IO, *Mem, MemSize.i)
  Protected *NCon.__NetConnection = *IO\handle
  If *NCon\BufferPos ; Buffer available
    If *NCon\BufferPos < MemSize
      MemSize = *NCon\BufferPos
    EndIf
    Protected NewMemSize = *NCon\BufferPos - MemSize
    ; Copy from internal Buffer to external
    CopyMemory(*NCon\Buffer, *Mem, MemSize)
    ; Is there something left in the Buffer?
    If Not NewMemSize
      FreeMemory(*NCon\Buffer)
      *NCon\Buffer = #Null
      *NCon\BufferPos = #Null
      *NCon\BufferSize = #Null
    Else ; Shrink Buffer
      MoveMemory(*NCon\Buffer + MemSize, *NCon\Buffer, NewMemSize)
      *NCon\BufferPos = NewMemSize
      *NCon\BufferSize = NewMemSize
      *NCon\Buffer = ReAllocateMemory(*NCon\Buffer, NewMemSize)
    EndIf
  EndIf
  ; Return Received Size
  ProcedureReturn MemSize
EndProcedure

; Network, IsAvailable
Procedure __IO_Network_IsAvailable(*IO.__IO)
  Protected *NCon.__NetConnection = *IO\handle
  Protected tReceive
  ; Is data Available=
  If (Not *NCon\IsConnection) Or (NetworkClientEvent(*NCon\Handle) = #PB_NetworkEvent_Data)
    Repeat
      ; Increase if necessary
      If *NCon\BufferSize < *NCon\BufferPos + *NCon\ReceiveSize
        *NCon\Buffer = ReAllocateMemory(*NCon\Buffer, *NCon\BufferPos + *NCon\ReceiveSize)
        *NCon\BufferSize = *NCon\BufferPos + *NCon\ReceiveSize
      EndIf
      ; Receive Data
      tReceive = ReceiveNetworkData(*NCon\Handle, *NCon\Buffer + *NCon\BufferPos, *NCon\ReceiveSize)
      If tReceive > 0
        *NCon\BufferPos + tReceive
      EndIf
    Until tReceive <> *NCon\ReceiveSize
  EndIf
  ; Return available data in Buffer
  ProcedureReturn *NCon\BufferPos
EndProcedure

; Network, Import
Procedure __IO_Network_ImportFromIO(*IO.__IO, *IO2.IO)
  If *IO2\IOOptionalFunctions() & #IO_ExportToIO
    *IO2\ExportToIO(*IO)
    ProcedureReturn #True
  EndIf
EndProcedure

; Network, Export
Procedure __IO_Network_ExportToIO(*IO.__IO, *IO2.IO)
  __IO_Network_IsAvailable(*IO)
  Protected *NCon.__NetConnection = *IO\handle
  If *NCon\BufferPos ; Buffer Vorhanden
    *IO2\Write(*NCon\Buffer, *NCon\BufferPos)
  EndIf
EndProcedure

; Flush Buffer
Procedure __IO_Network_Truncate(*IO.__IO)
  Protected *NCon.__NetConnection = *IO\handle
  If *NCon\Buffer
    FreeMemory(*NCon\Buffer)
    *NCon\Buffer = #Null
    *NCon\BufferPos = #Null
    *NCon\BufferSize = #Null
  EndIf
EndProcedure

; Close Connection
Procedure __IO_Network_Close(*IO.__IO)
  Protected *NCon.__NetConnection = *IO\handle
  __IO_Network_Truncate(*IO)
  CloseNetworkConnection(*NCon\Handle)
  ClearStructure(*IO, __IO)
  FreeMemory(*IO)
EndProcedure

; Network, Handle
Procedure __IO_Network_Handle(*IO.__IO)
  Protected *NCon.__NetConnection = *IO\handle
  ProcedureReturn *NCon\Handle
EndProcedure

; VTable: Network
DataSection
  __vTable_IO_Network:
  Data.i @__IO_Network_Write()
  Data.i @__IO_Network_Read()
  Data.i @__IO_Network_IsAvailable()
  Data.i @__IO_Network_Truncate()
  Data.i @__IO_Network_Close()
  Data.i @__IO_Network_Handle()
  Data.i @__IO_All_Name()
  Data.i @__IO_All_GetOptionalFunctions()
  Data.i 0 ; GetLenght
  Data.i 0 ; GetPosition
  Data.i 0 ; SetPosition
  Data.i @__IO_Network_ImportFromIO()
  Data.i @__IO_Network_ExportToIO()
EndDataSection
; IDE Options = PureBasic 5.11 (Windows - x86)
; CursorPosition = 138
; FirstLine = 127
; Folding = --
; EnableXP
; CPU = 1